from __future__ import annotations

from typing import Optional
from pathlib import Path

import cv2
import numpy as np

from .data_classes import RawImage


def blackout(raw_image: RawImage) -> np.ndarray:
    image = raw_image.get_image()

    for box in raw_image.bounding_boxes:
        cv2.rectangle(image, box, (0, 0, 0), -1)

    return image


def impaint(raw_image: RawImage, image: Optional[np.ndarray] = None) -> np.ndarray:
    image = image if image is not None else raw_image.get_image()
    
    # Create a mask where blacked-out areas are marked as 255 (white)
    mask = np.zeros(image.shape[:2], dtype=np.uint8)

    for (x, y, w, h) in raw_image.bounding_boxes:
        mask[y:y+h, x:x+w] = 255

    # Apply inpainting using the Telea method
    return cv2.inpaint(image, mask, inpaintRadius=3, flags=cv2.INPAINT_TELEA)


def pixelate(to_detect: str, debug_drawings: bool = True):
    raw_image = RawImage(to_detect)

    step_dir = raw_image.get_dir("steps")
    def write_image(image: np.ndarray, name: str):
        nonlocal debug_drawings
        
        f = str(step_dir / (name + raw_image.file.suffix))

        if debug_drawings:
            for box in raw_image.bounding_boxes:
                cv2.rectangle(image, box, (0, 255, 255), 1)
        
        cv2.imwrite(f, image)

    step_1 = blackout(raw_image)
    write_image(step_1, "step_1")

    step_2 = impaint(raw_image, image=step_1)
    write_image(step_2, "step_2")
